/*
-------------------------------------------------------------------------------
    This file is a part of the Vst Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: vpgxml.cpp
  Author: Francois Mazen
  Date: 28/05/07 16:25
  Description: class using Qt's XML functions in order to read and save the 
               lists of vst (.vpg) with their header's parameters
-------------------------------------------------------------------------------
*/

#include <QtGui>
#include "vpgxml.h"

//-----------------------------------------------------------------------------
VpgXml::VpgXml(QWidget * t_parent) : QAbstractListModel(t_parent)
{
      w_parent = t_parent;
}

//-----------------------------------------------------------------------------
void VpgXml::WriteFile(const char* filename)
{
     int i;
     int ret;
     QString Sbuffer;
     FxHeaderStruct *struct_temp;
     
     //create the dom (virtual) file
     QDomDocument doc;
     QDomElement root = doc.createElement( "vpg_vst_list" );
     doc.appendChild( root );
     
     for(i=0; i < dataList.size(); i++)
     {
         struct_temp = dataList.value(i);
         QDomElement cn = doc.createElement( "vst" );
         cn.setAttribute( "name", struct_temp->name );
         cn.setAttribute( "fx_id", struct_temp->fxID );
         cn.setAttribute( "fx_version", struct_temp->fxVersion );
         cn.setAttribute( "nb_param", struct_temp->nParams );
         
         root.appendChild(cn);
     }
     
     QFile file( filename );
     if( !file.open( QIODevice::WriteOnly ) )
     {
         //error while opening
         Sbuffer = tr("Could not opening the file: ");
         Sbuffer.append(filename);
         ret = QMessageBox::critical( w_parent, tr("Saving Vst List"), Sbuffer);
         return;
      }

      QTextStream ts( &file );
      ts << doc.toString();

      file.close();
     
}

//-----------------------------------------------------------------------------
void VpgXml::ReadFile(const char* filename)
{
     int ret;
     QDomDocument doc;
     QDomElement root;
     QDomNode n;
     bool error = false;
     FxHeaderStruct* fxhStruct;
     QVariant buffer;
     QString Sbuffer;
     
     //read the file
     QFile file( filename );
     if (!file.open(QIODevice::ReadOnly))
      {
         //error while opening
         Sbuffer = tr("Could not opening the file: ");
         Sbuffer.append(filename);
         ret = QMessageBox::critical( w_parent, tr("Opening Vst List"), Sbuffer);
         
         return;
      }
      
      
      if (!doc.setContent(&file)) 
      {
         file.close();
         return;
      }
     file.close();
     
     //check data
     root = doc.documentElement();
     if( root.tagName() != "vpg_vst_list" )
     {
         Sbuffer = tr("The file: ");
         Sbuffer.append(filename);
         Sbuffer.append(tr(" is not a valid vpg file"));
         ret = QMessageBox::critical( w_parent, tr("Opening Vst List"), Sbuffer);
         return;    
     }
     
     //clear the data:
     dataList.clear();
     //fill the data:
     n = root.firstChild();
     while( !n.isNull() )
     {
          QDomElement e = n.toElement();
          if( !e.isNull() )
          {
              if( e.tagName() == "vst" )
              {
                  //create a new header structure
                  fxhStruct = new FxHeaderStruct;
                  fxhStruct->fxID = e.attribute( "fx_id", "" );
                  buffer = e.attribute( "fx_version", "" );
                  fxhStruct->fxVersion = buffer.toInt();
                  fxhStruct->name = e.attribute( "name", "");
                  buffer = e.attribute( "nb_param", "");
                  fxhStruct->nParams = buffer.toInt();
                  
                  //stock the structure in our list:
                  dataList.append(fxhStruct);
              }
              else
              {
                  error = true;
              }
          }
          else 
          {
               error = true; //check error while reading
          } 
          n = n.nextSibling();
     }     
     
     if(error == true)
     {
         Sbuffer = tr("The program has dectected error in the file: ");
         Sbuffer.append(filename);
         ret = QMessageBox::information( w_parent, tr("Opening Vst List"), Sbuffer); 
     }
     
     //say to the list that the data have changed
     reset();
}


//-----------------------------------------------------------------------------
QVariant VpgXml::data(const QModelIndex & index, int role ) const
{
     //must return only the name of the header
     if(!index.isValid() || index.row() < 0 || index.row() >= dataList.size()+1)
       return QVariant();
     
     if(role == Qt::DisplayRole)
     {         
             return (QVariant(dataList.value(index.row())->name));
     }
     
     return QVariant();
     
}

//-----------------------------------------------------------------------------
QVariant VpgXml::headerData(int section, Qt::Orientation orientation, int role ) const
{
     return QVariant(); //nothing :)
}

//-----------------------------------------------------------------------------
int VpgXml::rowCount(const QModelIndex & parent ) const
{
    return dataList.size();
}


//-----------------------------------------------------------------------------
void VpgXml::addVst(FxHeaderStruct* fxstruct)
{
     dataList.append(fxstruct);    
     //the data have changed:             
     reset();   
}

//-----------------------------------------------------------------------------
void VpgXml::loadVst(const QModelIndex &index)
{
     emit validHeader(dataList.value(index.row()));
}
